# pylint: skip-file
from .base import Base
from .networks import Network
from .networkaddresses import NetworkAddress
from .networktokens import NetworkToken
from .networktransactions import NetworkTransaction
from .networktransfers import NetworkTransfer
from .smartcontracts import SmartContract


metadata = Base.metadata



"""Declares :class:`NetworkAddress`."""
import sqlalchemy
from sqlalchemy import Column
from sqlalchemy import BigInteger
from sqlalchemy import ForeignKey
from sqlalchemy import String
from sqlalchemy import Numeric
from sqlalchemy import UniqueConstraint
from sqlalchemy.dialects.postgresql import ARRAY
from sqlalchemy.dialects.postgresql import JSONB
from unimatrix.lib import timezone

from .base import Base
from .networks import Network


class NetworkAddress(Base):
    """Describes an address on a blockchain network that is of interest to
    us.
    """
    __tablename__ = 'networkaddresses'

    id = Column(
        BigInteger,
        primary_key=True,
        nullable=False,
        name='id'
    )

    network_id = Column(
        ForeignKey(Network.id),
        nullable=False,
        name='network_id'
    )

    address = Column(
        String,
        nullable=False,
        name='address'
    )

    label = Column(
        String,
        nullable=False,
        server_default=sqlalchemy.text("''"),
        default='',
        name='label'
    )

    imported = Column(
        BigInteger,
        nullable=False,
        default=timezone.now,
        server_default=sqlalchemy.text('0'),
        name='imported'
    )

    #: The list of events that should be observed for this address. Valid events
    #: are:
    #:
    #: ===============  ========================================================
    #: **Event**        **Description**
    #: ===============  ========================================================
    #: ``transaction``  A transaction i.e. a native token transfer or a smart
    #:                  contract invocation that does not transfer tokens.
    #: ``transfer``     Value transfers using standardized token protocols.
    #: ``balance``      Periodically inspect the balance of the address.
    #: ===============  ========================================================
    observe = Column(
        ARRAY(String),
        nullable=False,
        default=list,
        server_default=sqlalchemy.text("ARRAY[]::varchar[]"),
        name='observe'
    )

    #: The maximum balance in the networks' native token that this address
    #: may have before its marked as dirty. An address is dirty
    #: when an (unspecified) action has to be performed when
    #: a certain balance is present.
    max_balance = Column(
        Numeric,
        nullable=False,
        server_default=sqlalchemy.text('0'),
        default=0,
        name='max_balance'
    )

    annotations = Column(
        JSONB,
        nullable=False,
        server_default=sqlalchemy.text("'{}'"),
        default=dict,
        name='annotations'
    )

    __table_args__ = (
        UniqueConstraint('network_id', 'address'),
    )



"""Declares :class:`NetworkTransfer`."""
from sqlalchemy import Column
from sqlalchemy import BigInteger
from sqlalchemy import ForeignKey
from sqlalchemy import Numeric
from sqlalchemy import String

from .base import Base
from .networkaddresses import NetworkAddress


class NetworkTransfer(Base):
    __tablename__ = 'networktransfers'
    __module__ = 'ibrb.ext.meta.orm'

    id = Column(
        BigInteger,
        primary_key=True,
        nullable=False,
        name='id'
    )

    address_id = Column(
        ForeignKey(NetworkAddress.id),
        nullable=False,
        name='address_id'
    )

    transaction_id = Column(
        String,
        nullable=False,
        name='transaction_id'
    )

    block = Column(
        BigInteger,
        nullable=False,
        name='block'
    )

    timestamp = Column(
        BigInteger,
        nullable=False,
        name='timestamp'
    )

    contract = Column(
        String,
        nullable=False,
        name='contract'
    )

    sender = Column(
        String,
        nullable=False,
        name='sender'
    )

    receiver = Column(
        String,
        nullable=False,
        name='receiver'
    )

    value = Column(
        Numeric,
        nullable=False,
        name='value'
    )



"""Declares :class:`SmartContract`."""
from sqlalchemy import Column
from sqlalchemy import ForeignKey
from sqlalchemy import Integer
from sqlalchemy import String
from sqlalchemy import UniqueConstraint

from .base import Base
from .networks import Network


class SmartContract(Base):
    """Describes a smart contract on a blockchain network."""
    __tablename__ = 'smartcontracts'
    __module__ = 'ibrb.ext.meta.orm'

    #: A surrogate primary used for internal references.
    id = Column(
        Integer,
        primary_key=True,
        nullable=False,
        name='id'
    )

    #: References a :class:`Network` entity, identifying the blockchain network
    #: on which the contract lives.
    network_id = Column(
        ForeignKey(Network.id),
        nullable=False,
        name='network_id'
    )

    #: Specifies the protocol of the smart contract. The
    #: following protocols are supported:
    #:
    #: ==============================   ====================================
    #: **URN**                              **Description**
    #: ==============================   ====================================
    #: ``urn:ibrb:sc:unkown``           The protocol of the smart contract
    #:                                  is unknown
    #: ``urn:ibrb:sc:bep:20``           BEP-20 token on Binance Smart Chain
    #: ``urn:ibrb:sc:dc:pancake:2``     Pancakeswap 2 staking farm/pool
    #: ``urn:ibrb:sc:dc:uniswap``       Uniswap 2 staking farm/pool
    #: ``urn:ibrb:sc:erc:20``           ERC-20 token on Ethereum
    #: ``urn:ibrb:sc:lp:pancake:2``     Pancakeswap 2 liquidity pool
    #: ``urn:ibrb:sc:lp:uniswap``       Uniswap 2 liquidity pool
    #: ``urn:ibrb:sc:sol:token``        Solana Token Program
    #: ``urn:ibrb:sc:swap``             Unspecified token swapping contract
    #: ``urn:ibrb:sc:swap:pancake:2``   Pancakeswap 2
    #: ``urn:ibrb:sc:swap:uniswap``     Uniswap
    #: ==============================   ===================================
    protocol = Column(
        String,
        nullable=False,
        default='urn:ibrb:contract:unkown',
        name='protocol'
    )

    #: The address that created the contract.
    owner = Column(
        String,
        nullable=False,
        name='owner'
    )

    #: The contract address.
    address = Column(
        String,
        nullable=False,
        name='address'
    )

    #: A human-readable name.
    label = Column(
        String,
        nullable=False,
        default='',
        name='label'
    )

    __table_args__ = (
        UniqueConstraint('network_id', 'address',
            name='smartcontracts_unique_network_address'),
    )



"""Declares :class:`NetworkToken`."""
import decimal

from sqlalchemy import Column
from sqlalchemy import ForeignKey
from sqlalchemy import Integer
from sqlalchemy import Numeric
from sqlalchemy import String

from .smartcontracts import SmartContract


class NetworkToken(SmartContract):
    """Describes a token on a specific network."""
    __tablename__ = 'networktokens'
    __module__ = 'ibrb.ext.meta.orm'

    id = Column(
        ForeignKey(SmartContract.id),
        primary_key=True,
        nullable=False,
        name='id'
    )

    symbol = Column(
        String,
        nullable=False,
        name='symbol'
    )

    decimals = Column(
        Integer,
        nullable=False,
        name='decimals'
    )

    supply = Column(
        Numeric,
        nullable=False,
        default=decimal.Decimal('-1'),
        name='supply'
    )



"""Declares :class:`Network`."""
from sqlalchemy import Column
from sqlalchemy import SmallInteger
from sqlalchemy import String

from .base import Base


class Network(Base):
    """Maps numeric ids to network Uniform Resource Names (URNs)."""
    __tablename__ = 'networks'
    __module__ = 'ibrb.ext.meta.orm'

    id = Column(
        SmallInteger,
        primary_key=True,
        nullable=False,
        name='id'
    )

    protocol = Column(
        String,
        nullable=False,
        name='protocol'
    )

    urn = Column(
        String,
        unique=True,
        nullable=False,
        name='urn'
    )

    #: The preferred data source for a network client represented as a
    #: Data Source Name (DSN).
    dsn = Column(
        String,
        nullable=False,
        name='dsn'
    )




"""Declares :class:`NetworkTransaction`."""
from sqlalchemy import Column
from sqlalchemy import BigInteger
from sqlalchemy import ForeignKey
from sqlalchemy import Numeric
from sqlalchemy import String
from sqlalchemy import UniqueConstraint

from .base import Base
from .networkaddresses import NetworkAddress


class NetworkTransaction(Base):
    """Describes a transaction on a blockchain network."""
    __tablename__ = 'networktransactions'
    __module__ = 'ibrb.ext.meta.orm'

    address_id = Column(
        ForeignKey(NetworkAddress.id),
        primary_key=True,
        nullable=False,
        name='address_id'
    )

    transaction_id = Column(
        String,
        primary_key=True,
        nullable=False,
        name='transaction_id'
    )

    block = Column(
        BigInteger,
        nullable=False,
        name='block'
    )

    timestamp = Column(
        BigInteger,
        nullable=False,
        name='timestamp'
    )

    sender = Column(
        String,
        nullable=False,
        name='sender'
    )

    receiver = Column(
        String,
        nullable=False,
        name='receiver'
    )

    #: The cost of the transaction, in the networks' native token.
    cost = Column(
        Numeric,
        nullable=False,
        name='cost'
    )

    #: The amount of the networks' native token transferred, if any.
    amount = Column(
        Numeric,
        nullable=False,
        name='amount'
    )

    __table_args__ = (
        UniqueConstraint('address_id', 'transaction_id'),
    )
